"use strict";
/**
 * 数据库相关操作
 * Created at 2020-10-15 17:28
 *
 * @author Wheeler https://github.com/WheelerLee
 * @copyright 2020 Activatortube, INC.
 *
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const typeorm_1 = require("typeorm");
const change_case_1 = require("change-case");
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const FSUtils_1 = __importDefault(require("./FSUtils"));
/**
 * 类型的映射关系，主要是mysql数据类型和typescript的映射关系
 */
const VAR_MAPPER = [
    {
        varType: 'string',
        dbType: ['char', 'varchar', 'tinytext', 'text', 'mediumtext', 'longtext'],
    },
    {
        varType: 'number',
        dbType: ['tinyint', 'smallint', 'mediumint', 'int', 'integer', 'bigint', 'float', 'double'],
    }
];
/**
 * 应该忽略的公共字段
 */
const IGNORE_COLUMN = ['id', 'deleted', 'createdAt', 'updatedAt', 'sortedNum', 'creater'];
class SqlUtils {
    /**
     * 连接数据库，返回连接实例
     * @param info 数据库连接信息
     */
    static connect(info) {
        return __awaiter(this, void 0, void 0, function* () {
            let connection = typeorm_1.createConnection(info);
            return connection;
        });
    }
    /**
     * 通过数据库的数据类型获取ts的数据类型
     * @param dbType 数据库数据类型
     */
    static getVarTypeByDBType(dbType) {
        for (const item of VAR_MAPPER) {
            if (item.dbType.indexOf(dbType) >= 0) {
                return item.varType;
            }
        }
        return undefined;
    }
    /**
     * 获取单表的唯一约束，如果是联合约束，将会用逗号将多个字段分开
     */
    static getUniqueIndexs(connection, database, tableName) {
        return __awaiter(this, void 0, void 0, function* () {
            const uniqueIndexs = [[], []];
            const allIndexs = yield connection.manager.query(`SHOW INDEX FROM ${database}.${tableName}`);
            const uniqueMap = {};
            for (const index of allIndexs) {
                if (index.Non_unique === '0') { // 表示有唯一约束
                    // uniqueIndexs.push(index.Column_name);
                    if (uniqueMap[index.Key_name]) {
                        uniqueMap[index.Key_name].push(index.Column_name);
                    }
                    else {
                        uniqueMap[index.Key_name] = [index.Column_name];
                    }
                }
            }
            for (const key of Object.keys(uniqueMap)) {
                if (uniqueMap[key].length === 1) {
                    uniqueIndexs[0].push(uniqueMap[key][0]);
                }
                else {
                    uniqueIndexs[1].push(uniqueMap[key].map((a) => `'${a}'`).join(', '));
                }
            }
            return uniqueIndexs;
        });
    }
    static getForeignKey(connection, database, tableName) {
        return __awaiter(this, void 0, void 0, function* () {
            const columUsages = yield connection.manager
                .query('SELECT * FROM information_schema.KEY_COLUMN_USAGE WHERE TABLE_SCHEMA=? AND TABLE_NAME=?', [database, tableName]);
            const foreignKeys = {};
            for (const columUsage of columUsages) {
                if (columUsage.REFERENCED_TABLE_NAME) { // 关联的表
                    foreignKeys[columUsage.COLUMN_NAME] = columUsage.REFERENCED_TABLE_NAME;
                }
            }
            return foreignKeys;
        });
    }
    /**
     * 获取实体的具体信息
     * @param connection 数据库连接
     * @param database 数据库
     * @param tableName 表名
     */
    static getEntitieInfo(connection, database, tableName) {
        return __awaiter(this, void 0, void 0, function* () {
            const cases = tableName.split('_');
            let modelName; // 生产的model名
            let moduleName; // 模块名，会按照模块生成不同的文件夹
            if (cases.length === 1) { // 不带_的表名，推断为不带前缀没有模块的
                modelName = change_case_1.pascalCase(tableName);
                moduleName = '';
            }
            else {
                moduleName = cases[0];
                modelName = change_case_1.pascalCase(cases.splice(1).join(' '));
            }
            const uniqueIndexs = yield this.getUniqueIndexs(connection, database, tableName);
            const foreignKeys = yield this.getForeignKey(connection, database, tableName);
            const dbColumns = yield connection.manager.query('SELECT * FROM `INFORMATION_SCHEMA`.`COLUMNS` WHERE '
                + '(`TABLE_SCHEMA` =? AND `TABLE_NAME`=?)', [database, tableName]);
            const columns = [];
            for (const dbColumn of dbColumns) {
                // 忽略的字段
                if (IGNORE_COLUMN.indexOf(dbColumn.COLUMN_NAME) >= 0) {
                    continue;
                }
                // console.log(dbColumn);
                const varType = this.getVarTypeByDBType(dbColumn.DATA_TYPE);
                let columnType = dbColumn.COLUMN_TYPE;
                columnType = columnType.replace(/[^0-9]/ig, '');
                const length = parseInt(columnType);
                const column = {
                    entityName: foreignKeys[dbColumn.COLUMN_NAME] ?
                        change_case_1.camelCase(foreignKeys[dbColumn.COLUMN_NAME].split('_').splice(1).join(' '))
                        :
                            change_case_1.camelCase(dbColumn.COLUMN_NAME),
                    name: dbColumn.COLUMN_NAME,
                    dataType: dbColumn.DATA_TYPE,
                    type: dbColumn.COLUMN_TYPE,
                    varType: varType,
                    comment: dbColumn.COLUMN_COMMENT,
                    nullable: dbColumn.IS_NULLABLE === 'YES',
                    unique: uniqueIndexs[0].indexOf(dbColumn.COLUMN_NAME) >= 0,
                    foreignKey: foreignKeys[dbColumn.COLUMN_NAME] ?
                        {
                            moduleName: foreignKeys[dbColumn.COLUMN_NAME].split('_')[0],
                            name: change_case_1.pascalCase(foreignKeys[dbColumn.COLUMN_NAME].split('_').splice(1).join(' ')),
                            columns: [],
                            tableName: foreignKeys[dbColumn.COLUMN_NAME]
                        }
                        :
                            undefined
                };
                if (length) {
                    if (varType === 'string') {
                        column.length = length;
                    }
                    else if (varType === 'number') {
                        column.width = length;
                    }
                }
                if (dbColumn.COLUMN_DEFAULT) {
                    column.default = dbColumn.COLUMN_DEFAULT;
                }
                columns.push(column);
            }
            return {
                name: modelName,
                tableName: tableName,
                columns: columns,
                moduleName: moduleName,
                varName: change_case_1.camelCase(modelName),
                compositeUnique: uniqueIndexs[1],
                hasManyToOne: Object.keys(foreignKeys).length > 0,
                hasColumn: columns.length > Object.keys(foreignKeys).length
            };
        });
    }
    /**
     * 获取所有的实体类
     * @param connection 数据库连接
     * @param database 数据库
     */
    static getEntities(connection, database) {
        return __awaiter(this, void 0, void 0, function* () {
            const tables = yield connection.manager.query('SELECT * FROM `INFORMATION_SCHEMA`.'
                + '`TABLES` WHERE `TABLE_SCHEMA`=?', [database]);
            let es = [];
            for (const table of tables) {
                es.push(yield this.getEntitieInfo(connection, database, table.TABLE_NAME));
            }
            return es;
        });
    }
    static genaterEntity(entitiesPath, entity) {
        return __awaiter(this, void 0, void 0, function* () {
            let entityPath = entitiesPath;
            if (entity.moduleName) {
                entityPath = path_1.default.resolve(entitiesPath, entity.moduleName);
            }
            yield fs_extra_1.default.ensureDir(entityPath);
            FSUtils_1.default.translate(path_1.default.resolve(__dirname, '../../templates/to/entity/Entity.ejs'), `${entityPath}/${entity.name}.ts`, entity);
            // const template = await fs.readFile(path.resolve(__dirname, '../../templates/to/entity/Entity.ejs'));
            // const model = ejs.render(template.toString(), entity);
            // await fs.writeFile(`${entityPath}/${entity.name}.ts`, Buffer.from(model), { flag: 'w', encoding: 'utf8' });
        });
    }
    /**
     * 根据实体信息生成所有的实体类文件
     * @param entities 所有的实体集合
     */
    static genaterEntities(entities) {
        return __awaiter(this, void 0, void 0, function* () {
            let entitiesPath = path_1.default.resolve(process.cwd(), 'api/entities');
            yield fs_extra_1.default.ensureDir(entitiesPath);
            for (const entity of entities) {
                yield this.genaterEntity(entitiesPath, entity);
            }
            fs_extra_1.default.copy(path_1.default.resolve(__dirname, '../../templates/to/entity/BaseModel.tmpl'), path_1.default.resolve(entitiesPath, 'BaseModel.ts'));
        });
    }
}
exports.default = SqlUtils;
